using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.EnergeticaNatura.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.EnergeticaNatura.BusinessObjects.Definitions.Tables;
using System.Drawing.Printing;
using System.Runtime.InteropServices;
using System.Threading;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Foundation.Messages;
//css_ref System.Drawing.dll;
//css_ref System.ServiceModel.dll;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesInvoiceCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesInvoiceCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<Collection<DocumentRef>> DocRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>

        /// <summary>
        /// Definitions
        /// </summary>
        private static class Definitions
        {
            /// <summary>
            /// Rembours
            /// </summary>
            public const string REMBOURS = "5";
            public const string REMBOURS_BE = "22";

            /// <summary>
            /// Cheque
            /// </summary>
            public const string CHEQUE = "6";

            /// <summary>
            /// Visa
            /// </summary>
            public const string VISA = "7";

            //report definitions
            public const string REPORT_PATH_LABEL = @"LabelBPostDE.rpt";
            public const int NUMBER_OF_LABELS = 1;

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesInvoiceCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesInvoiceCreatedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            Dictionary<string, PmxParameterValue> tcParameters = null;
            string query = null;
            DataSet dsShippingType = null;
            string labelType = null;
            string msgKey = null;
            ProdumexError pmxError = null;
            Message msg = null;

            int user = sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null, delegate(PmxDbConnection dbConn, object[] parameters)
            {
                return dbConn.InternalKeySboUser;
            });


            tcParameters = sboProviderService.InvokeMethodWithDbConnection<Dictionary<string, PmxParameterValue>>(false, true, null, null, new DelegateWithPmxDbConnection<Dictionary<string, PmxParameterValue>>(ExecuteGetParameters), session.DeviceInstanceId);

            foreach (DocumentRef docRef in DocRefs.Get())
            {
                if (docRef.DocType == ((int)SboObjectType.SalesInvoice).ToString())
                {
                    try
                    {
                        query = BuildQuery.GetQueryGetShippingTypeOnHeaderForSalesInvoice(docRef.DocEntry);
                        dsShippingType = sboProviderService.RunView(false, null, null, query);

                        if (this.GetNumberOfRows(dsShippingType) > 0)
                        {
                            labelType = dsShippingType.Tables[0].Rows[0][PmxShippingTypeENTable.Columns.LabelType.NAME] as string;

                            switch (labelType)
                            {
                                case PmxShippingTypeLabelTypeColumnValues.BPost.VALUE:

                                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", null,
                                       "NoTranslationOfMessageKey", true,
                                       "MessageKey", "Aanmaken label voor BPost ..."));
                                    sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null, CreateBPostFile, session.DeviceInstanceId, tcParameters, docRef.DocEntry, user);

                                    if (countryCode.ToUpper() == "DE" || countryCode.ToUpper() == "GB" || countryCode.ToUpper() == "LU")
                                    {
                                    Step_PrintLabel:
                                        #region Step_PrintLabel

                                        string reportPath = Definitions.REPORT_PATH_LABEL;
                                        sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), docRef.DocEntry, session.DeviceInstanceId, reportPath);
                                        #endregion


                                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                            "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                                            "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

                                        msg = WaitForMessage();

                                        if (msg.Name.EndsWith(".Yes"))
                                        {
                                            goto Step_PrintLabel;
                                        }
                                    }
                                    break;

                                case PmxShippingTypeLabelTypeColumnValues.DPD.VALUE:
                                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", null,
                                       "NoTranslationOfMessageKey", true,
                                       "MessageKey", "Aanmaken label voor DPD ..."));

                                    sboProviderService.InvokeMethodWithDbConnection<bool>(false, true, null, null, CreateDPDFile, session.DeviceInstanceId, tcParameters, docRef.DocEntry);

                                    break;
                                case PmxShippingTypeLabelTypeColumnValues.LaPoste.VALUE:
                                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", null,
                                       "NoTranslationOfMessageKey", true,
                                       "MessageKey", "Aanmaken label voor La Poste ..."));

                                    sboProviderService.InvokeMethodWithDbConnection<bool>(false, true, null, null, CreateLaPosteFile, session.DeviceInstanceId, tcParameters, docRef.DocEntry);

                                    break;
                                default:
                                    break;
                            }//switch


                        }

                    }
                    catch (ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        msgKey = ex.Message;
                        pmxError = ex.Error;
                    }
                    catch (Exception ex)
                    {
                        s_log.Error("", ex);
                        msgKey = ex.Message;
                    }

                    if (msgKey != null)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", null,
                                "MessageKey", msgKey,
                                "PmxError", pmxError,
                                "ShowButton", true));
                        WaitForMessage();
                    }



                }





            }

            //wait, to avoid problems on wave
            Thread.Sleep(500);

            return;

        }

        #endregion


        #region HELPERS


        private string countryCode = null;
        private string m_bpostFileName = null;
        private string m_bpostPrinter = null;


        /// <summary>
        /// Creates the B post file.
        /// </summary>
        /// <param name="dbConn">The db conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private bool CreateBPostFile(PmxDbConnection dbConn, object[] parameters)
        {
            string deviceInstanceId = (string)parameters[0];
            int docEntry = (int)parameters[2];
            int user = (int)parameters[3];


            PmxParameterValueProvider parProv = new PmxParameterValueProvider(dbConn);
            Dictionary<string, PmxParameterValue> tcParameters = (Dictionary<string, PmxParameterValue>)parameters[1];



            string labelFilePath = PmxParameterValueProvider.GetParameterValue<string>(tcParameters, PmxExtensionParameterCodesEN.BPOST_FILE_PATH);
            string printer = PmxParameterValueProvider.GetParameterValue<string>(tcParameters, PmxExtensionParameterCodesEN.BPOST_PRINTER);



            string strLabelFileText = null;
            using (TextReader reader = new StreamReader(labelFilePath))
            {
                strLabelFileText = reader.ReadToEnd();
            }




            string invoiceNumber = null;



            string query = BuildQuery.GetSboSalesInvoiceHeader(docEntry, dbConn.DbCultureInfo);
            using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
            {
                if (rs.EoF)
                    return false;  // 

                invoiceNumber = rs.GetTypedValue<int>(PmxSalesInvoiceTable.Columns.DocNum.NAME).ToString();

                countryCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CountryS.NAME);
                string street = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetS.NAME);
                string streetNo = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoS.NAME);
                string block = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BlockS.NAME);
                string building = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BuildingS.NAME);
                string zipCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeS.NAME);
                string city = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CityS.NAME);

                if (string.IsNullOrEmpty(countryCode))
                {
                    countryCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CountryB.NAME);
                    street = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetB.NAME);
                    streetNo = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoB.NAME);
                    block = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BlockB.NAME);
                    building = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BuildingB.NAME);
                    zipCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeB.NAME);
                    city = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CityB.NAME);
                }

		    countryCode = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(countryCode));
                street = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(street));
                streetNo = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(streetNo));
                block = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(block));
                building = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(building));
                zipCode = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(zipCode));
                city = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(city));


                int lLabelRegel = 100;

                if (countryCode != "BE" &&
                    countryCode != "FR" &&
                    countryCode != "LU" &&
                    countryCode != "NL")
                {
                    StringBuilder labelFileText = new StringBuilder();
                    //Address label


                    labelFileText.AppendFormat("{0}L{1}", ((char)2).ToString(), Environment.NewLine);
                    labelFileText.AppendFormat("D11{0}", Environment.NewLine);

                    string aanhef = rs.GetTypedValue<string>(PmxBusinessPartnerENTable.Columns.Aanhef.NAME);
                    if (!string.IsNullOrEmpty(aanhef))
                    {
                        aanhef += " ";
                    }

                    labelFileText.AppendFormat("49110060050{0}{1}{2}{3}", lLabelRegel.ToString().PadLeft(4, '0'), aanhef, rs.GetTypedValue<string>(PmxSalesInvoiceENTable.Columns.CardName.NAME), Environment.NewLine);

                    lLabelRegel += 70;

                    labelFileText.AppendFormat("49110060050{0}{1} {2}{3}", lLabelRegel.ToString().PadLeft(4, '0'), street, streetNo, Environment.NewLine);
                    if (!string.IsNullOrEmpty(block) &&
                        !string.IsNullOrEmpty(building))
                    {
                        lLabelRegel += 70;
                        labelFileText.AppendFormat("49110060050{0}{1} {2}{3}", lLabelRegel.ToString().PadLeft(4, '0'), block, building, Environment.NewLine);
                    }
                    lLabelRegel += 70;
                    labelFileText.AppendFormat("49110060050{0}{1}-{2} {3}{4}", lLabelRegel.ToString().PadLeft(4, '0'), countryCode, zipCode, city, Environment.NewLine);

                    labelFileText.AppendFormat("Q0001{0}E{0}", Environment.NewLine);


                    strLabelFileText = labelFileText.ToString();

                }

                if (countryCode == "BE")
                {

                    #region BE - LU

                    string rembours = null;
                    string invoiceRembours = rs.GetTypedValue<int>(SboSalesInvoiceTable.Columns.PaymentTermsGroupNum.NAME).ToString();
                    if (invoiceRembours == Definitions.REMBOURS || invoiceRembours == Definitions.REMBOURS_BE)
                    {
                        //rembours is the total of the document
                        rembours = rs.GetTypedValue<double>(SboSalesInvoiceTable.Columns.DocTotal.NAME).ToString();
                    }

                    string cardCode = rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.CardCode.NAME);


                    string strProduct = "031";
                    string strEuro = "0000";
                    string strCent = "00";

                    if (string.IsNullOrEmpty(rembours))
                    {
                        strProduct = "030";
                    }
                    else
                    {
                        string[] amount = rembours.Split(new char[] { ',', '.' });

                        strEuro = amount[0].PadLeft(4, '0');

                        if (amount.Length > 1)
                        {
                            strCent = amount[1].PadRight(2, '0');
                        }

                    }

                    string strID = "3232048790" + invoiceNumber.PadLeft(4, ' ').Substring(invoiceNumber.PadLeft(4, ' ').Length - 4, 4) + cardCode + strProduct;




                    string strBarcode1 = cardCode.PadLeft(4, '0').Substring(cardCode.PadLeft(4, '0').Length - 4, 4) + strEuro + strCent + zipCode;
                    string strBarcode2 = strID;





                    if (string.IsNullOrEmpty(rembours))
                    {
                        strLabelFileText = strLabelFileText.Replace("<TEKST1>", "");
                        strLabelFileText = strLabelFileText.Replace("<TEKST2>", "Paquet - Pakket");
                        strLabelFileText = strLabelFileText.Replace("<COD1>", "");
                        strLabelFileText = strLabelFileText.Replace("<COD2>", "");
                        strLabelFileText = strLabelFileText.Replace("<COD3>", "");
                        strLabelFileText = strLabelFileText.Replace("<BARCODE1>", strBarcode2);
                        strLabelFileText = strLabelFileText.Replace("<BARCODE2>", "");
                        strLabelFileText = strLabelFileText.Replace("<TBARCODE2>", "");
                        strLabelFileText = strLabelFileText.Replace("<TEKST3>", "");
                        strLabelFileText = strLabelFileText.Replace("<TEKST4>", "");
                        strLabelFileText = strLabelFileText.Replace("<TEKST5>", "");
                        strLabelFileText = strLabelFileText.Replace("<TEKST6>", "");
                        strLabelFileText = strLabelFileText.Replace("<REKENING>", "");
                        strLabelFileText = strLabelFileText.Replace("<BEDRAG>", "");
                    }
                    else
                    {
                        strLabelFileText = strLabelFileText.Replace("<TEKST1>", "");
                        strLabelFileText = strLabelFileText.Replace("<TEKST2>", "Envoi contre remboursement - Verrekenzending");
                        strLabelFileText = strLabelFileText.Replace("<COD1>", "2911S0103650345P010P010COD");
                        strLabelFileText = strLabelFileText.Replace("<COD2>", "1X1100003200340P00100010350037003800340");
                        strLabelFileText = strLabelFileText.Replace("<BARCODE1>", strBarcode2);
                        strLabelFileText = strLabelFileText.Replace("<BARCODE2>", "2e1504004500295C" + strBarcode1);
                        strLabelFileText = strLabelFileText.Replace("<TBARCODE2>", strBarcode1);
                        strLabelFileText = strLabelFileText.Replace("<TEKST3>", "2911S0104600140P006P006Montant");
                        strLabelFileText = strLabelFileText.Replace("<TEKST4>", "2911S0104600126P006P006Bedrag");
                        strLabelFileText = strLabelFileText.Replace("<TEKST5>", "2911S0102800140P006P006Numero de compte");
                        strLabelFileText = strLabelFileText.Replace("<TEKST6>", "2911S0102800126P006P006Rekeningnummer");
                        strLabelFileText = strLabelFileText.Replace("<REKENING>", "BE25 7310 4159 4682");
                        strLabelFileText = strLabelFileText.Replace("<BEDRAG>", "EUR " + rembours);
                    }

                    string name = rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.ShipToCode.NAME);
                    if (string.IsNullOrEmpty(name))
                    {
                        name = rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.CardName.NAME);
                    }

			  name = Encoding.ASCII.GetString(Encoding.GetEncoding("Cyrillic").GetBytes(name));

                    strLabelFileText = strLabelFileText.Replace("<NAAM>", name);
                    strLabelFileText = strLabelFileText.Replace("<ADRES>", string.Format("{0} {1}", street, streetNo));
                    strLabelFileText = strLabelFileText.Replace("<PLAATS>", string.Format("{0} {1}", zipCode, city));


                    //remove the BE- from the zip code
                    zipCode = zipCode.Substring(zipCode.IndexOf('-') + 1, zipCode.Length - (zipCode.IndexOf('-') + 1));


                    //if (zipCode[0] > '3' &&
                    //    zipCode[0] < '8')
                    //{
                    //    strLabelFileText = strLabelFileText.Replace("<ZONE>", "C");
                    //}
                    //else
                    //{
                    //    strLabelFileText = strLabelFileText.Replace("<ZONE>", "A");
                    //}

                    //check numeric value
                    int result = 0;
                    if (int.TryParse(zipCode.Trim()[0].ToString(), out result))
                    {
                        /*Postcodes beginnend met 2, 3 een A
                         Postcodes beginnend met 1, 4, 5, 6, 7 een C
                         Postcodes beginnend met 8, 9 een GENT X*/
                        if (result == 2 ||
                            result == 3)
                        {
                            strLabelFileText = strLabelFileText.Replace("<ZONE>", "A");
                        }
                        else if (result == 8 ||
                            result == 9)
                        {
                            strLabelFileText = strLabelFileText.Replace("<ZONE>", "GENT X");
                        }
                        else
                        {
                            strLabelFileText = strLabelFileText.Replace("<ZONE>", "C");
                        }
                    }
                    else
                    {
                        s_log.WarnFormat("First digit of zipcode {0} is not numeric!", zipCode);
                    }
                    #endregion
                }


                if (countryCode.ToUpper() == "DE" || countryCode.ToUpper() == "GB" || countryCode.ToUpper() == "LU")
                {
                    //Set to NULL, because we do not actually want to create the file
                    invoiceNumber = null;

                    string barcode = CalculateShippingBarcode(dbConn, docEntry);
                    PmxSalesInvoiceProvider invProv = new PmxSalesInvoiceProvider((SboConnection)dbConn);
                    using (PmxSalesInvoice inv = invProv.GetBO(docEntry))
                    {
                        inv[PmxSalesInvoiceENTable.Columns.ShippingBarcode.NAME] = barcode;

                        invProv.UpdateBO(inv, false, false);
                    }

                    //query = BuildQuery.SetShippingBarcode(docEntry, barcode);
                    //SboRecordsetHelper.RunQueryNoReturn(s_log, query, dbConn);
                }


            }


            if (!string.IsNullOrEmpty(invoiceNumber))
            {
                try
                {
                    //create temp file 



                    // Write file
                    string outputDir = @"C:\Produmex\";

                    // Make sure path ends with back-slash
                    if (!outputDir.EndsWith(@"\"))
                    {
                        outputDir += @"\";
                    }


                    System.IO.Directory.CreateDirectory(outputDir);



                    m_bpostFileName = Path.Combine(outputDir, string.Format("BPost_{0}_{1}_{2}.prn", deviceInstanceId, user, docEntry));




                    using (StreamWriter writer = new StreamWriter(m_bpostFileName, false, Encoding.UTF8))
                    {
                        writer.Write(strLabelFileText);
                        writer.Flush();
                    }


                    m_bpostPrinter = printer;

                    //print file to printer

                    ThreadStart threadStart = new ThreadStart(PrintBPostFile);
                    Thread thread = new Thread(threadStart);
                    thread.Start();




                    //Print file


                    //using (PrintDocument prnDocument = new PrintDocument())
                    //{
                    //    //Get the default printer name.                
                    //    //prnDocument = new PrintDocument();
                    //    prnDocument.PrinterSettings.PrinterName = printer;

                    //    prnDocument.PrintPage += delegate(object sender, PrintPageEventArgs e)
                    //    {
                    //        System.Drawing.Font fnt = new System.Drawing.Font(System.Drawing.FontFamily.GenericSerif, 10);
                    //        e.Graphics.DrawString(strLabelFileText, fnt, System.Drawing.Brushes.Black, 0, 0);
                    //    };

                    //    prnDocument.Print();
                    //}

                }
                catch (COMException comException)
                {
                    //Log the exception 
                    s_log.Error("", comException);
                    throw;
                }
                catch (Exception sysException)
                {
                    //Log the exception                
                    s_log.Error("", sysException);
                    throw;
                }

                return true;
            }
            else
            {
                s_log.WarnFormat("Unable to create label file for BPost for docEntry {0}. No invoice was found.", docEntry);
                return false;
            }
        }

        /// <summary>
        /// Calculates the shipping barcode
        /// </summary>
        /// <param name="dbConn">The db conn</param>
        private string CalculateShippingBarcode(PmxDbConnection dbConn, int docEntry)
        {
            string result = "";
            string serviceIndicator = "CD"; //fixed
            string countryCodeBarcode = "BE"; //fixed
            string serial = "";
            int currentValue = 0;

            string query = BuildQuery.GetShippingBarcodeCurrentValue();

            using (ISboRecordset rsLine = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
            {
                while (!rsLine.EoF)
                {
                    currentValue = rsLine.GetTypedValue<int>(PmxShippingBarcodeENTable.Columns.CurrentValue.NAME);
                    rsLine.MoveNext();
                }
            }

            serial = currentValue.ToString("D8");

            int checkDigit = CalculateCheckDigit(serial);

            query = BuildQuery.SetShippingBarcodeCurrentValue();
            SboRecordsetHelper.RunQueryNoReturn(s_log, query, dbConn);

            result = serviceIndicator + serial + checkDigit + countryCodeBarcode;
            return result;
        }

        /// <summary>
        /// Calculates the check digit
        /// </summary>
        /// <param name="serial">The serial number that needs the check digit</param>
        /// <returns>The check digit</returns>
        private int CalculateCheckDigit(string serial)
        {
            int[] weights = { 8, 6, 4, 2, 3, 5, 9, 7 };
            int totalWeight = 0;
            int result = 0;

            for (int i = 0; i < serial.ToString().Length; i++)
            {
                totalWeight += weights[i] * int.Parse(serial.ToString().Substring(i, 1));
            }

            result = totalWeight % 11;
            result = 11 - result;

            switch (result)
            {
                case 10:
                    result = 0;
                    break;
                case 11:
                    result = 5;
                    break;
                default:
                    break;
            }
            return result;
        }


        /// <summary>
        /// Prints the B post file.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <param name="printer">The printer.</param>
        private void PrintBPostFile()
        {

            try
            {

                s_log.WarnFormat("Printing file {0} to printer {1}", m_bpostFileName, m_bpostPrinter);

                string sCmdLineNET = "NET USE " + m_bpostPrinter + " /HOME";
                string sCmdLine = "COPY " + m_bpostFileName + " " + m_bpostPrinter;

                s_log.WarnFormat("Using command {0}", sCmdLine);

                // * Declare and instantiate a new process component. *
                System.Diagnostics.Process processCmd = new System.Diagnostics.Process();

                processCmd.StartInfo.FileName = "CMD.exe";
                processCmd.StartInfo.UseShellExecute = false;
                processCmd.StartInfo.CreateNoWindow = true;
                processCmd.StartInfo.RedirectStandardInput = true;
                processCmd.StartInfo.RedirectStandardOutput = true;
                // * Launch the process *
                processCmd.Start();
                // * Retrieve the input/outp t stream *
                StreamWriter swInput = processCmd.StandardInput;
                StreamReader srOutput = processCmd.StandardOutput;
                // * Send the command *
                swInput.AutoFlush = true;
                swInput.Write(sCmdLine + System.Environment.NewLine);
                swInput.Flush();
                swInput.Write(System.Environment.NewLine);
                System.Threading.Thread.Sleep(30000);
                swInput.Write("exit" + System.Environment.NewLine);
                // * Retrieve the result *
                string sOutput = srOutput.ReadToEnd();
                // * The '/C' Tells windows to run the command then terminate *
                //System.Diagnostics.Process.Start( "CMD.exe", "/C "+sCmdLine ) ;
                processCmd.Close();

            }
            catch (Exception ex)
            {
                s_log.Error("PRINT BPOST FILE", ex);
                throw;
            }



        }


        /// <summary>
        /// Creates the DPD file.
        /// </summary>
        /// <param name="dbConn">The db conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private bool CreateDPDFile(PmxDbConnection dbConn, object[] parameters)
        {
            string deviceInstanceId = (string)parameters[0];
            int docEntry = (int)parameters[2];

            PmxParameterValueProvider parProv = new PmxParameterValueProvider(dbConn);
            Dictionary<string, PmxParameterValue> tcParameters = (Dictionary<string, PmxParameterValue>)parameters[1];

            StringBuilder sbDoc = new StringBuilder();

            string invoiceNumber = null;

            string query = BuildQuery.GetSboSalesInvoiceHeader(docEntry, dbConn.DbCultureInfo);
            using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
            {
                if (rs.EoF)
                    return false;  // 

                //NP;<Naam klant>;<Straat>;<Land>;<Postcode>;<Plaats>;<FactuurNr> 




                invoiceNumber = rs.GetTypedValue<int>(PmxSalesInvoiceTable.Columns.DocNum.NAME).ToString();

                string cardName = rs.GetTypedValue<string>(PmxSalesInvoiceTable.Columns.CardName.NAME);

                //TBA added
                if (!string.IsNullOrEmpty(rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.ShipToCode.NAME)))
                {
                    cardName = rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.ShipToCode.NAME);
                }


                countryCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CountryS.NAME);
                string street = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetS.NAME);
                string streetNo = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoS.NAME);
                string block = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BlockS.NAME);
                string building = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BuildingS.NAME);
                string zipCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeS.NAME);
                string city = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CityS.NAME);

                if (string.IsNullOrEmpty(countryCode))
                {
                    countryCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CountryB.NAME);
                    street = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetB.NAME);
                    streetNo = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoB.NAME);
                    block = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BlockB.NAME);
                    building = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BuildingB.NAME);
                    zipCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeB.NAME);
                    city = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CityB.NAME);
                }


                //TBA change: check if we find an e-mail linked to the BP
                string shippingEmail = rs.GetTypedValue<string>("U_U_ShippingEmail");

                if (string.IsNullOrEmpty(shippingEmail))
                {
                    shippingEmail = rs.GetTypedValue<string>("E_Mail");
                }

                string shipType = "NP";
                string messageType = null;
                string contactData = null;
                string proactiveRule = null;
                string proactiveLanguage = null;

                if (!string.IsNullOrEmpty(shippingEmail))
                {
                    shipType = "NCP, PRO";
                    messageType = "E";
                    contactData = shippingEmail;
                    proactiveRule = "904";
                    proactiveLanguage = "NL";
                }



                if (cardName != null && street != null && countryCode != null && zipCode != null && city != null)
                {
                    //remove country code from zip code
                    //NL-1234 => 1234
                    zipCode = zipCode.Substring(zipCode.IndexOf('-') + 1, zipCode.Length - (zipCode.IndexOf('-') + 1));


                   // sbDoc.AppendFormat("{0};{1}; {2};{3};{4};{5};{6};{7};{8};{9};{10};{11};{12}",
                   //     shipType, cardName, street, streetNo, countryCode, zipCode, city, invoiceNumber, shippingEmail, messageType, contactData, proactiveRule, proactiveLanguage);
                     sbDoc.AppendFormat("{0};{1}; {2};{3};{4};{5};{6};{7};{8};{9};{10};{11}",
                         shipType, cardName, street,  countryCode, zipCode, city, invoiceNumber, shippingEmail, messageType, contactData, proactiveRule, proactiveLanguage);
                }
                else
                {
                    s_log.WarnFormat("Unable to create label file for DPD for docEntry {0}. Missing data .", docEntry);
                }
            }

            if (!string.IsNullOrEmpty(invoiceNumber))
            {

                // Write file
                string outputDir = PmxParameterValueProvider.GetParameterValue<string>(tcParameters, PmxExtensionParameterCodesEN.DPD_OUT_DIR);

                // Make sure path ends with back-slash
                if (!outputDir.EndsWith(@"\"))
                {
                    outputDir += @"\";
                }

                string outputDirTemp = string.Format("{0}Temp", outputDir);

                System.IO.Directory.CreateDirectory(outputDir);
                System.IO.Directory.CreateDirectory(outputDirTemp);



                string filename = string.Format("{0}.txt", invoiceNumber);


                using (StreamWriter writer = new StreamWriter(Path.Combine(outputDirTemp, filename), false, Encoding.GetEncoding(1252)))
                {
                    writer.Write(sbDoc.ToString());
                    writer.Flush();
                }

                //if file exists in destination, we remove it
                if (File.Exists(System.IO.Path.Combine(outputDir, filename)))
                {
                    File.Delete(System.IO.Path.Combine(outputDir, filename));
                }

                //move created file to correct folder
                System.IO.File.Move(System.IO.Path.Combine(outputDirTemp, filename), System.IO.Path.Combine(outputDir, filename));


                return true;
            }
            else
            {
                s_log.WarnFormat("Unable to create label file for DPD for docEntry {0}. No invoice was found.", docEntry);
                return false;
            }
        }



        /// <summary>
        /// Creates the la poste file.
        /// </summary>
        /// <param name="dbConn">The db conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private bool CreateLaPosteFile(PmxDbConnection dbConn, object[] parameters)
        {
            string deviceInstanceId = (string)parameters[0];
            int docEntry = (int)parameters[2];

            PmxParameterValueProvider parProv = new PmxParameterValueProvider(dbConn);
            Dictionary<string, PmxParameterValue> tcParameters = (Dictionary<string, PmxParameterValue>)parameters[1];

            StringBuilder sbDoc = new StringBuilder();

            string invoiceNumber = null;

            string query = BuildQuery.GetSboSalesInvoiceHeader(docEntry, dbConn.DbCultureInfo);
            using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
            {
                if (rs.EoF)
                    return false;  // 

                invoiceNumber = rs.GetTypedValue<int>(PmxSalesInvoiceTable.Columns.DocNum.NAME).ToString();

                sbDoc.AppendFormat("EXP{0}{1}",
                    invoiceNumber.ToString().PadRight(35, ' '),
                    AddSpaces(8));

                countryCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CountryS.NAME);
                string street = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetS.NAME);
                string streetNo = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoS.NAME);
                string block = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BlockS.NAME);
                string building = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BuildingS.NAME);
                string zipCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeS.NAME);
                string city = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CityS.NAME);

                if (string.IsNullOrEmpty(countryCode))
                {
                    countryCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CountryB.NAME);
                    street = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetB.NAME);
                    streetNo = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoB.NAME);
                    block = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BlockB.NAME);
                    building = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.BuildingB.NAME);
                    zipCode = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeB.NAME);
                    city = rs.GetTypedValue<string>(SboSalesInvoiceTaxExtensionTable.Columns.CityB.NAME);
                }

                zipCode = zipCode.Substring(zipCode.IndexOf('-') + 1, zipCode.Length - (zipCode.IndexOf('-') + 1));


                bool overZee = false;

                if (countryCode == "FR" &&
                    (zipCode.StartsWith("97") || zipCode.StartsWith("98")))
                {
                    overZee = true;
                }

                string rembours = "";

                if (overZee)
                {
                    sbDoc.Append("COM ");
                }
                else
                {
                    string invoiceRembours = rs.GetTypedValue<int>(SboSalesInvoiceTable.Columns.PaymentTermsGroupNum.NAME).ToString();


                    if (invoiceRembours == Definitions.REMBOURS || invoiceRembours == Definitions.REMBOURS_BE)
                    {
                        //rembours is the total of the document
                        rembours = rs.GetTypedValue<double>(SboSalesInvoiceTable.Columns.DocTotal.NAME).ToString().Replace(",", ".");
                        sbDoc.Append("COL ");
                    }
                    else
                    {
                        rembours = "";
                        sbDoc.Append("COLD");
                    }


                }

                //6 + 17 spaces
                sbDoc.Append(AddSpaces(23));

                string customerName = rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.CardName.NAME);

                if (!string.IsNullOrEmpty(rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.ShipToCode.NAME)))
                {
                    customerName = rs.GetTypedValue<string>(SboSalesInvoiceTable.Columns.ShipToCode.NAME);
                }

                sbDoc.Append(customerName.Trim().PadRight(35, ' '));

                if (countryCode != "FR")
                {
                    sbDoc.AppendFormat(string.Format("{0} {1}", street, streetNo).PadRight(35, ' '));
                    sbDoc.Append(AddSpaces(35));
                }
                else
                {
                    sbDoc.AppendFormat(string.Format("{0} {1}", street, streetNo).PadRight(35, ' '));
                    sbDoc.AppendFormat(string.Format("{0} {1}", block, building).PadRight(35, ' '));
                }

                //35 spaces
                sbDoc.Append(AddSpaces(35));
                sbDoc.Append(zipCode.PadRight(9, ' '));



                //End of first line

                sbDoc.Append(city.PadRight(35, ' '));
                sbDoc.AppendFormat("{0} ", countryCode);

                sbDoc.Append(AddSpaces(70));
                sbDoc.Append(AddSpaces(1));
                sbDoc.Append(AddSpaces(5));
                sbDoc.Append(AddSpaces(6));

                sbDoc.AppendFormat("{0}", rs.GetTypedValue<double>(SboSalesInvoiceTable.Columns.Weight.NAME).ToString().Replace(",", ".").PadRight(15, ' '));
                sbDoc.Append(rembours.PadRight(15, ' '));
                sbDoc.Append(AddSpaces(3));

                string verzekering = rs.GetTypedValue<double>(PmxSalesInvoiceENTable.Columns.Verzekering.NAME).ToString().Replace(",", ".");

                sbDoc.Append(ConvertToFixedLength(verzekering, 15));

                sbDoc.Append(AddSpaces(3));
                sbDoc.Append("N");
                sbDoc.Append(AddSpaces(17));
                sbDoc.Append(AddSpaces(25));
                sbDoc.Append(AddSpaces(4));


                //End line 2

                sbDoc.Append("0NN2");

                //Add E-mail adress
                string emailAdress = rs.GetTypedValue<string>("E_Mail");
                if (!string.IsNullOrEmpty(emailAdress))
                {
                    sbDoc.Append(ConvertToFixedLength(emailAdress, 75));
                }
                else
                {
                    sbDoc.Append(AddSpaces(75));
                }


                sbDoc.Append("4");
                sbDoc.Append(AddSpaces(25));
                sbDoc.Append(AddSpaces(25));
                sbDoc.Append(invoiceNumber.PadRight(25, ' '));
                sbDoc.Append(AddSpaces(4));

                //End line 3

                //added to cope with adding of telephone number
                sbDoc.Append(AddSpaces(35));
                sbDoc.Append(AddSpaces(35));
                sbDoc.Append(AddSpaces(35));
                sbDoc.Append(AddSpaces(35));
                sbDoc.Append(AddSpaces(11));
                sbDoc.Append(AddSpaces(27));
                sbDoc.Append(AddSpaces(1));
                sbDoc.Append(AddSpaces(29));
                sbDoc.Append(AddSpaces(38));

                bool useCellular = true;
                string cellular = rs.GetTypedValue<string>("Cellular");
                int cellularAsInt = 0;
                string phone1 = rs.GetTypedValue<string>(SboBusinessPartnerTable.Columns.Phone1.NAME);
                int phone1AsInt = 0;
                //char[] charsToTrim = { '.', ' ', '\'', '/','-','_', '(', ')', '+' };

                if (!string.IsNullOrEmpty(cellular))
                {
                    // start with replacing +33 with 0
                    if (cellular.StartsWith("+33"))
                    {
                        cellular = cellular.Replace("+33", "0");
                    }

                    //cellular = cellular.Trim(charsToTrim);

                    cellular = cellular.Replace(".", string.Empty);
                    cellular = cellular.Replace(" ", string.Empty);
                    cellular = cellular.Replace("/", string.Empty);
                    cellular = cellular.Replace("\'", string.Empty);
                    cellular = cellular.Replace("(", string.Empty);
                    cellular = cellular.Replace(")", string.Empty);
                    cellular = cellular.Replace("-", string.Empty);
                    cellular = cellular.Replace("_", string.Empty);
                    cellular = cellular.Replace("+", string.Empty);

                    if (!int.TryParse(cellular, out cellularAsInt))
                    {
                        // not numeric so we take Phone1
                        cellular = null;
                        useCellular = false;
                    }
                    else
                    {
                        //make sure the length is 10 and take the 10 most 'right' characters

                        cellular = cellular.PadRight(10, ' ').Substring(cellular.PadRight(10, ' ').Length - 10, 10);
                    }
                }
                else
                {
                    useCellular = false;
                    // no cellular found, so we take Phone1
                    if (!string.IsNullOrEmpty(phone1))
                    {
                        // start with replacing +33 with 0
                        if (phone1.StartsWith("+33"))
                        {
                            phone1 = phone1.Replace("+33", "0");
                        }

                        //phone1 = phone1.Trim(charsToTrim);
                        phone1 = phone1.Replace(".", string.Empty);
                        phone1 = phone1.Replace(" ", string.Empty);
                        phone1 = phone1.Replace("/", string.Empty);
                        phone1 = phone1.Replace("\'", string.Empty);
                        phone1 = phone1.Replace("(", string.Empty);
                        phone1 = phone1.Replace(")", string.Empty);
                        phone1 = phone1.Replace("-", string.Empty);
                        phone1 = phone1.Replace("_", string.Empty);
                        phone1 = phone1.Replace("+", string.Empty);

                        if (!int.TryParse(phone1, out phone1AsInt))
                        {
                            // not numeric so we do not set anything
                            phone1 = null;
                        }
                        else
                        {
                            //make sure the length is 10 and take the 10 most 'right' characters
                            phone1 = phone1.PadRight(10, ' ').Substring(phone1.PadRight(10, ' ').Length - 10, 10);
                        }
                    }
                }

                if (useCellular)
                {
                    sbDoc.Append(ConvertToFixedLength(cellular, 10));
                }
                else
                {
                    sbDoc.Append(ConvertToFixedLength(phone1, 10));
                }


                //End line 4

                if (overZee)
                {
                    sbDoc.Append(Environment.NewLine);

                    //get all lines
                    string queryLines = BuildQuery.GetSboSalesInvoiceLines(docEntry, dbConn.DbCultureInfo);
                    using (ISboRecordset rsLines = SboRecordsetHelper.RunQuery(s_log, queryLines, dbConn))
                    {

                        while (!rsLines.EoF)
                        {

                            //strExp = strExp & "CN2"
                            //strExp = strExp & strOmschrijving & Space(25 - Len(strOmschrijving))
                            //strExp = strExp & Space(3 - Len(strAantal)) & strAantal
                            //strExp = strExp & strGewicht & Space(5 - Len(strGewicht))
                            //strExp = strExp & strTotaal & Space(8 - Len(strTotaal))
                            //strExp = strExp & strStatnr
                            //strExp = strExp & "NL"

                            string itemName = rsLines.GetTypedValue<string>(PmxSalesInvoiceLineTable.Columns.Description.NAME);
                            sbDoc.AppendFormat("CN2{0}", ConvertToFixedLength(rsLines.GetTypedValue<string>(PmxSalesInvoiceLineTable.Columns.Description.NAME), 25));

                            sbDoc.Append(rsLines.GetTypedValue<double>(PmxSalesInvoiceLineTable.Columns.Quantity.NAME).ToString().Replace(",", ".").PadLeft(3, ' '));
                            sbDoc.Append(rsLines.GetTypedValue<double>(PmxSalesInvoiceLineTable.Columns.Weight1.NAME).ToString().Replace(",", ".").PadRight(5, ' '));
                            sbDoc.Append(rsLines.GetTypedValue<double>(PmxSalesInvoiceLineTable.Columns.LineTotal.NAME).ToString().Replace(",", ".").PadRight(8, ' '));
                            sbDoc.Append(rsLines.GetTypedValue<string>(PmxItemENTable.Columns.StatNR.NAME));
                            sbDoc.Append("NL");

                            rsLines.MoveNext();

                            if (!rsLines.EoF)
                            {
                                //sbDoc.Append(Environment.NewLine);
                            }
                        }//while
                    }
                }




            }

            if (!string.IsNullOrEmpty(invoiceNumber))
            {

                // Write file
                string outputDir = PmxParameterValueProvider.GetParameterValue<string>(tcParameters, PmxExtensionParameterCodesEN.LAPOSTE_OUT_DIR);

                // Make sure path ends with back-slash
                if (!outputDir.EndsWith(@"\"))
                {
                    outputDir += @"\";
                }

                string outputDirTemp = string.Format("{0}Temp", outputDir);

                System.IO.Directory.CreateDirectory(outputDir);
                System.IO.Directory.CreateDirectory(outputDirTemp);



                string filename = string.Format("{0}.txt", invoiceNumber);


                using (StreamWriter writer = new StreamWriter(Path.Combine(outputDirTemp, filename), false, Encoding.GetEncoding(1252)))
                {
                    writer.Write(sbDoc.ToString());
                    writer.Flush();
                }

                //if file exists in destination, we remove it
                if (File.Exists(System.IO.Path.Combine(outputDir, filename)))
                {
                    File.Delete(System.IO.Path.Combine(outputDir, filename));
                }

                //move created file to correct folder
                System.IO.File.Move(System.IO.Path.Combine(outputDirTemp, filename), System.IO.Path.Combine(outputDir, filename));


                return true;
            }
            else
            {
                s_log.WarnFormat("Unable to create label file for La Poste for docEntry {0}. No invoice was found.", docEntry);
                return false;
            }
        }


        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }


        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <param name="dbConn">The db conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private Dictionary<string, PmxParameterValue> ExecuteGetParameters(PmxDbConnection dbConn, object[] parameters)
        {
            string deviceInstanceId = (string)parameters[0];

            PmxParameterValueProvider parProv = new PmxParameterValueProvider(dbConn);
            return parProv.GetParameterValuesAsDictionary(String.Format("{0}->{1}", deviceInstanceId, PmxPropertyCodes.OSE_PARAMETERSET), true, false);

        }


        /// <summary>
        /// Adds the spaces.
        /// </summary>
        /// <param name="count">The count.</param>
        /// <returns></returns>
        private string AddSpaces(int count)
        {
            StringBuilder sb = new StringBuilder();
            for (int iLoop = 0; iLoop < count; iLoop++)
            {
                sb.Append(" ");
            }
            return sb.ToString();
        }

        /// <summary>
        /// Converts the length of to fixed.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="length">The length.</param>
        /// <param name="padding">The padding.</param>
        /// <returns></returns>
        private string ConvertToFixedLength(string value, int length, char padding)
        {
            if (value == null)
            {
                value = "";
            }
            return value.Substring(0, Math.Min(length, value.Length)).PadRight(length, padding);
        }


        /// <summary>
        /// Converts the length of to fixed.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="length">The length.</param>
        /// <param name="padding">The padding.</param>
        /// <returns></returns>
        private string ConvertToFixedLength(string value, int length)
        {
            return ConvertToFixedLength(value, length, ' ');
        }

        /// <summary>
        /// Executes the print label.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintLabel(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@invoiceDocEntry", (int)parameters[0]));
            //reportParameters.Add(new ReportParameter("@colli", (string)parameters[1]));

            return prov.PrintReport(reportParameters, (string)parameters[2], Definitions.NUMBER_OF_LABELS, (string)parameters[1], null, false);
        }


        #endregion


    }

    #region BUILD QUERY

    /// <summary>
    /// Static class grouping our native queries
    /// </summary>
    internal static class BuildQuery
    {
        /// <summary>
        /// Gets the query get shipping type on header for sales invoice.
        /// </summary>
        /// <param name="docEntry">The doc entry.</param>
        /// <returns></returns>
        public static string GetQueryGetShippingTypeOnHeaderForSalesInvoice(int docEntry)
        {
            CultureInfo dbCultureInfo = CultureInfo.InvariantCulture;

            string shippingTypeTable1Name = "OSHP1";
            string shippingTypeTable2Name = "OSHP2";

            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT DISTINCT ");

            strBuilder.AppendFormat(" ISNULL( {0}, ISNULL({1}, 'N' )) AS {2} ",
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable1Name, PmxShippingTypeTable.Columns.AutomaticShipping.NAME),
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable2Name, PmxShippingTypeTable.Columns.AutomaticShipping.NAME),
            SqlCommandHelper.EscapeColumnName(PmxShippingTypeTable.Columns.AutomaticShipping.NAME));

            strBuilder.AppendFormat(", ISNULL( {0}, ISNULL({1}, 'N' )) AS {2} ",
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable1Name, PmxShippingTypeTable.Columns.AutomaticInvoicing.NAME),
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable2Name, PmxShippingTypeTable.Columns.AutomaticInvoicing.NAME),
            SqlCommandHelper.EscapeColumnName(PmxShippingTypeTable.Columns.AutomaticInvoicing.NAME));

            strBuilder.AppendFormat(", ISNULL( {0}, {1}) AS {2} ",
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable1Name, PmxShippingTypeTable.Columns.Code.NAME),
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable2Name, PmxShippingTypeTable.Columns.Code.NAME),
            SqlCommandHelper.EscapeColumnName(PmxShippingTypeTable.Columns.Code.NAME));

            strBuilder.AppendFormat(", ISNULL( {0}, {1}) AS {2} ",
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable1Name, PmxShippingTypeTable.Columns.Name.NAME),
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable2Name, PmxShippingTypeTable.Columns.Name.NAME),
            SqlCommandHelper.EscapeColumnName(PmxShippingTypeTable.Columns.Name.NAME));


            strBuilder.AppendFormat(", ISNULL( {0}, {1}) AS {2} ",
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable1Name, PmxShippingTypeENTable.Columns.LabelType.NAME),
            SqlCommandHelper.EscapeTableColumnName(shippingTypeTable2Name, PmxShippingTypeENTable.Columns.LabelType.NAME),
            SqlCommandHelper.EscapeColumnName(PmxShippingTypeENTable.Columns.LabelType.NAME));



            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesInvoiceTable.NAME));

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesInvoiceLineTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesInvoiceLineTable.NAME, PmxSalesInvoiceLineTable.Columns.DocEntry.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocEntry.NAME);


            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboShippingTypeTable.NAME));
            strBuilder.AppendFormat(" {0} ON ", SqlCommandHelper.EscapeTableName(shippingTypeTable1Name));
            SqlCommandHelper.AddTableColumnName(strBuilder, shippingTypeTable1Name, SboShippingTypeTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.TrnspCode.NAME);

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboShippingTypeTable.NAME));
            strBuilder.AppendFormat(" {0} ON ", SqlCommandHelper.EscapeTableName(shippingTypeTable2Name));
            SqlCommandHelper.AddTableColumnName(strBuilder, shippingTypeTable2Name, SboShippingTypeTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.TrnsCode.NAME);


            strBuilder.Append(" WHERE ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocEntry.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, dbCultureInfo));


            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets the sbo sales invoice header.
        /// </summary>
        /// <param name="docEntry">The doc entry.</param>
        /// <param name="cultInfo">The cult info.</param>
        /// <returns></returns>
        internal static string GetSboSalesInvoiceHeader(int docEntry, CultureInfo cultInfo)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("SELECT {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.DocNum.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.CardName.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.CardCode.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.NumAtCard.NAME));




            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.StreetS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.StreetB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.StreetS.NAME));

            strBuilder.AppendFormat(", {0}, {1}",
              SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.StreetNoS.NAME),
              SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.StreetNoB.NAME),
              SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.StreetNoS.NAME));
            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.ZipCodeS.NAME));
            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.CityS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.CityB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.CityS.NAME));
            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.StateS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.StateB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.StateS.NAME));
            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.CountryS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.CountryB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.CountryS.NAME));
            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.BlockS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.BlockB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.BlockS.NAME));
            strBuilder.AppendFormat(", {0}, {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.BuildingS.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.BuildingB.NAME),
                SqlCommandHelper.EscapeColumnName(SboSalesInvoiceTaxExtensionTable.Columns.BuildingS.NAME));

            strBuilder.AppendFormat(", {0}",
               SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, "Cellular"));

            strBuilder.AppendFormat(", {0}",
               SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.Phone1.NAME));

            strBuilder.AppendFormat(", {0}",
               SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, "E_Mail"));
            strBuilder.AppendFormat(", {0}",
          SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, "U_U_ShippingEmail"));


            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.DocCurrency.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.DocTotal.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.Weight.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, PmxSalesInvoiceENTable.Columns.Verzekering.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, PmxBusinessPartnerENTable.Columns.Aanhef.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.PaymentTermsGroupNum.NAME));

            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.ShipToCode.NAME));

            strBuilder.AppendFormat("\r\nFROM {0}",
                SqlCommandHelper.EscapeTableName(SboSalesInvoiceTable.NAME));

            strBuilder.AppendFormat("\r\nINNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(SboSalesInvoiceTaxExtensionTable.NAME));
            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTaxExtensionTable.NAME, SboSalesInvoiceTaxExtensionTable.Columns.DocEntry.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.DocEntry.NAME));

            strBuilder.AppendFormat("\r\nINNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.CardCode.NAME));

            strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
                SqlCommandHelper.EscapeTableName(SboPaymentTermsTypeTable.NAME));
            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(SboPaymentTermsTypeTable.NAME, SboPaymentTermsTypeTable.Columns.GroupNum.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.PaymentTermsGroupNum.NAME));

            strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
                SqlCommandHelper.EscapeTableName(SboContactPersonTable.NAME));
            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(SboContactPersonTable.NAME, SboContactPersonTable.Columns.CntctCode.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.CntctCode.NAME));

            strBuilder.AppendFormat("\r\nWHERE {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.DocEntry.NAME),
                SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, cultInfo));


            return strBuilder.ToString();
        }



        /// <summary>
        /// Gets the sbo sales invoice header.
        /// </summary>
        /// <param name="docEntry">The doc entry.</param>
        /// <param name="cultInfo">The cult info.</param>
        /// <returns></returns>
        internal static string GetSboSalesInvoiceLines(int docEntry, CultureInfo cultInfo)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("SELECT {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.ItemCode.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.Description.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.Weight1.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.Quantity.NAME));
            strBuilder.AppendFormat(", {0}",
               SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.LineTotal.NAME));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnName(PmxItemENTable.NAME, PmxItemENTable.Columns.StatNR.NAME));



            strBuilder.AppendFormat("\r\nFROM {0}",
                SqlCommandHelper.EscapeTableName(SboSalesInvoiceLineTable.NAME));

            strBuilder.AppendFormat("\r\nINNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.ItemCode.NAME));

            //strBuilder.AppendFormat("\r\nINNER JOIN {0}",
            //    SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
            //strBuilder.AppendFormat(" ON {0} = {1}",
            //    SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME),
            //    SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.CardCode.NAME));

            //strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
            //    SqlCommandHelper.EscapeTableName(SboPaymentTermsTypeTable.NAME));
            //strBuilder.AppendFormat(" ON {0} = {1}",
            //    SqlCommandHelper.EscapeTableColumnName(SboPaymentTermsTypeTable.NAME, SboPaymentTermsTypeTable.Columns.GroupNum.NAME),
            //    SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.PaymentTermsGroupNum.NAME));

            //strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
            //    SqlCommandHelper.EscapeTableName(SboContactPersonTable.NAME));
            //strBuilder.AppendFormat(" ON {0} = {1}",
            //    SqlCommandHelper.EscapeTableColumnName(SboContactPersonTable.NAME, SboContactPersonTable.Columns.CntctCode.NAME),
            //    SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceTable.NAME, SboSalesInvoiceTable.Columns.CntctCode.NAME));

            strBuilder.AppendFormat("\r\nWHERE {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(SboSalesInvoiceLineTable.NAME, SboSalesInvoiceLineTable.Columns.DocEntry.NAME),
                SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, cultInfo));


            return strBuilder.ToString();
        }

        /// <summary>
        /// Get the current value of the shipping barcode serial
        /// </summary>
        /// <returns>The current serial number</returns>
        internal static string GetShippingBarcodeCurrentValue()
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("SELECT {0} FROM {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxShippingBarcodeENTable.NAME, PmxShippingBarcodeENTable.Columns.CurrentValue.NAME),
                SqlCommandHelper.EscapeTableName(PmxShippingBarcodeENTable.NAME));

            return strBuilder.ToString();
        }

        /// <summary>
        /// Set the current value of the shipping barcode serial
        /// </summary>
        internal static string SetShippingBarcodeCurrentValue()
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("UPDATE {0} SET {1} = {2} + 1",
                SqlCommandHelper.EscapeTableName(PmxShippingBarcodeENTable.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxShippingBarcodeENTable.NAME, PmxShippingBarcodeENTable.Columns.CurrentValue.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxShippingBarcodeENTable.NAME, PmxShippingBarcodeENTable.Columns.CurrentValue.NAME));
            //SqlCommandHelper.EscapeColumnValue(PmxShippingBarcodeENTable.Columns.CurrentValue.NAME));

            return strBuilder.ToString();
        }

        /// <summary>
        /// Set the current value of the shipping barcode serial
        /// </summary>
        internal static string SetShippingBarcode(int docEntry, string value)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("UPDATE {0} SET {1} = '{2}' WHERE {3} = {4}",
                SqlCommandHelper.EscapeTableName(PmxSalesInvoiceTable.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceENTable.Columns.ShippingBarcode.NAME),
                SqlCommandHelper.EscapeColumnValue(value),
                SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocEntry.NAME),
                SqlCommandHelper.EscapeColumnValue(docEntry));

            return strBuilder.ToString();
        }
    }


    #endregion
}